#include <allegro.h>

int inited=0;
int loaded=0;
int quitloop=0;

const int xres=640;
const int yres=480;
const int bpp=16;
const int lps=64;

BITMAP *dbuffer=NULL;

volatile long speed_counter=0;

void increment_speed_counter()
{
	speed_counter++;
}
END_OF_FUNCTION(increment_speed_counter); 

void quit(){
	if(inited) { allegro_exit(); --inited; }	
}

int init(){
	int status=-1;
	if((status=allegro_init())) return status;
	if((status=install_timer())) return status;
	if((status=install_keyboard())) return status;
	set_color_depth(bpp);
	if((status=set_gfx_mode(GFX_AUTODETECT_WINDOWED,xres,yres,0,0))) return status;
	clear_bitmap(screen); 
 	++inited;
	return 0;
}

void unload(){
	if(dbuffer) {destroy_bitmap(dbuffer); dbuffer=NULL;}
	if(loaded) --loaded;
}

int load(){
	int status=-1;
	dbuffer=create_bitmap(xres,yres);
	if(!dbuffer) return status;
	++loaded;	
	return 0;
}

int i;
int ring=0;

int x1=25,y1=250,sp1=8,sgnx1=1,sgny1=1;
int x2=350,y2=35,sp2=7,sgnx2=1,sgny2=-1;
int px1[16],py1[16];
int px2[16],py2[16];

int col,col2,colscale,lpsscale;
int r[16]={255,255,255,255,255,255,173,  8,  8,  8,  8,  8, 41,140,206,255};
int g[16]={ 12,101,101,158,211,255,223,207,207,142,142, 81, 12, 12, 12, 12};
int b[16]={  0,  0,  0,  0,  0,  0,  8, 16, 16,206,206,206,206,206,123,  0};
int ringcol[16][16];

void do_input(){
	if(key[KEY_ESC]) ++quitloop;
}

void do_logic(){	
	x1+=sp1*sgnx1;
	y1+=sp1*sgny1;
	x2+=sp2*sgnx2;
	y2+=sp2*sgny2;

	if(x1<0) { x1=-x1; sgnx1=-sgnx1; }
	if(x1>=xres) { x1=(2*xres)-x1; sgnx1=-sgnx1; }

	if(x2<0) { x2=0; sgnx2=-sgnx2; }
	if(x2>=xres) { x2=(2*xres)-x2; sgnx2=-sgnx2; }

	if(y1<0) { y1=0; sgny1=-sgny1; }
	if(y1>=yres) { y1=(2*yres)-y1; sgny1=-sgny1; }

	if(y2<0) { y2=0; sgny2=-sgny2; }
	if(y2>=yres) { y2=(2*yres)-y2; sgny2=-sgny2; }	

	px1[ring]=x1; py1[ring]=y1;
	px2[ring]=x2; py2[ring]=y2;
	ring++; ring&=15;
}

void do_render(){	
	col2=col>>5;
	for(i=0;i<16;++i){
		fastline(dbuffer,px1[ring],py1[ring],px2[ring],py2[ring],ringcol[col2][i]);
		ring++; ring&=15;
	}
	col++; col&=511;
}

void looprun(){
	while(!quitloop){	
		do_input();
   		while(speed_counter > 0){
			do_logic();
			speed_counter--;
		}
		do_render();		
		blit(dbuffer,screen,0,0,0,0,xres,yres);
		clear_bitmap(dbuffer);
 	}
}

void loopquit(){
}

void loopinit(){
	int j;
	LOCK_VARIABLE(speed_counter);
	LOCK_FUNCTION(increment_speed_counter);
	install_int_ex(increment_speed_counter, BPS_TO_TIMER(lps));
	clear_bitmap(dbuffer); 
	quitloop=0;
	for(i=0;i<16;i++) { 
		px1[i]=x1; py1[i]=y1; 
		px2[i]=x2; py2[i]=y2; 
		for(j=0;j<16;j++) { 
			ringcol[j][i]=makecol((r[j]*(i+1))>>4,(g[j]*(i+1))>>4,(b[j]*(i+1))>>4);
		}
	}
	col=0; col2=0; colscale=lps<<3; 
}

int main(){
	if(init()) quit();
	if(load()) unload();
	loopinit();
	looprun();
	loopquit();
	unload();
	quit();	
	return 0;
}
END_OF_MAIN();